package weaver.hrm.pm.action;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import com.api.hrm.bean.HrmFieldBean;
import com.api.hrm.service.HrmResourceWorkService;
import com.api.hrm.util.ServiceUtil;
import com.engine.common.service.impl.HrmCommonServiceImpl;
import com.engine.hrm.entity.RuleCodeType;
import com.engine.hrm.util.CodeRuleManager;
import com.engine.hrm.util.face.HrmFaceCheckManager;
import com.engine.hrm.util.face.ValidateFieldManager;
import com.engine.hrm.util.face.bean.CheckItemBean;
import org.apache.commons.validator.EmailValidator;
import weaver.common.DateUtil;
import weaver.common.StringUtil;
import weaver.conn.RecordSet;
import weaver.conn.RecordSetTrans;
import weaver.docs.docs.CustomFieldManager;
import weaver.file.ImageFileManager;
import weaver.general.GCONST;
import weaver.general.PasswordUtil;
import weaver.general.Util;
import weaver.hrm.common.DbFunctionUtil;
import weaver.hrm.company.DepartmentComInfo;
import weaver.hrm.companyvirtual.DepartmentVirtualComInfo;
import weaver.hrm.definedfield.HrmFieldManager;
import weaver.hrm.finance.SalaryManager;
import weaver.hrm.passwordprotection.manager.HrmResourceManager;
import weaver.hrm.resource.CustomFieldTreeManager;
import weaver.hrm.resource.ResourceComInfo;
import weaver.hrm.settings.ChgPasswdReminder;
import weaver.hrm.settings.RemindSettings;
import weaver.hrm.tools.HrmDateCheck;
import weaver.system.SysRemindWorkflow;
import weaver.systeminfo.SysMaintenanceLog;
import weaver.systeminfo.SystemEnv;

/**
 * 人员入职
 * @author wcd
 * @version 1.0 2015-07-07
 */
public class HrmResourceEntrantAction extends PmAction {
	private Map<String, Object> baseData = new HashMap<>();
	private Map<String, Object> custData = new HashMap<>();
	private Map<Integer, List> hrmSelectItem = new HashMap<>();
	private Map<Integer, List> cusSelectItem = new HashMap<>();
	private Map<String, Map> fieldMap = new HashMap<>();

	private static List donotExportFields = Arrays.asList(new String[]{"loginid", "workyear", "companyworkyear"});
	private static List<String> parseList = new ArrayList(Arrays.asList("jobtitle", "locationid", "status", "accounttype", "managerid", "jobcall", "joblevel", "assistantid"));

	protected int todo() {
		return HRM_RESOURCE_ENTRANT;
	}

	protected void parse() throws Exception {
		try {
			writeLog("in HrmResourceEntrantAction parse " + todo());
			RecordSet rs = new RecordSet();

			findFields();
			loadHrmSelectItem();
			loadCusSelectItem();

			for (String key : baseData.keySet()) {
				if (parseList.contains(key))
					baseData.put(key, Util.getIntValue(rsdt.getString(key), 0));
				else if (key.equals("systemlanguage"))
					baseData.put(key, Util.getIntValue(rsdt.getString(key), 7));
				else
					baseData.put(key, transValue(key));
			}

			for (String key : custData.keySet()) {
				custData.put(key, transValue(key));
			}

			//处理人员照片
			String resourceimageid = (String) getValue("resourceimageid");
			resourceimageid = getImageIdByDocid(resourceimageid);
			//考虑到流程文档权限的问题，把附件复制一份
			resourceimageid = Util.null2String(ImageFileManager.copyImageFile(Util.getIntValue(resourceimageid)));
			if (resourceimageid.equals("0")) {
				writeLog("resourceimageid为0，上传图片没有成功");
			}

			//处理密码
			String password = (String) getValue("password");
			ChgPasswdReminder reminder = new ChgPasswdReminder();
			RemindSettings settings = reminder.getRemindSettings();
			//判断是否开启了【启用初始密码】
			String defaultPasswordEnable = Util.null2String(settings.getDefaultPasswordEnable());
			//【初始密码】
			String defaultPassword = Util.null2String(settings.getDefaultPassword());
			//如果入职流程的密码为空。并且开启了【启用初始密码】，且初始密码不为空，则默认取初始密码作为密码
			if (password.equals("") && defaultPasswordEnable.equals("1") && !defaultPassword.equals("")) {
				password = defaultPassword;
			}

			//处理电话号码
			String mobile = (String) getValue("mobile");
			String telephone = (String) getValue("telephone");
			CheckItemBean mobileBean = new CheckItemBean("mobile", mobile);
			ValidateFieldManager.validate(mobileBean);
			if (!mobileBean.isPass()) {
				this.reqManager.setMessage(mobileBean.getCheckMsg());
				throw new Exception(mobileBean.getCheckMsg());
			}
			CheckItemBean telephoneBean = new CheckItemBean("telephone", telephone);
			ValidateFieldManager.validate(telephoneBean);
			if (!telephoneBean.isPass()) {
				this.reqManager.setMessage(telephoneBean.getCheckMsg());
				throw new Exception(telephoneBean.getCheckMsg());
			}

			//处理姓名
			String belongto = (String) getValue("belongto");
			String lastname = (String) getValue("lastname");
			writeLog("in HrmResourceEntrantAction lastname " + lastname + ":" + ((lastname.length() == 0)));
			if (lastname.length() == 0) {
				this.reqManager.setMessage(SystemEnv.getHtmlLabelName(125311,user.getLanguage()));
				throw new Exception("姓名不能为空！");
			}

			// 基本信息
			int accounttype = Util.getIntValue(Util.null2String(getValue("accounttype")), 0);
			baseData.remove("accounttype");
			if (accounttype == 0) belongto = "-1";
			String departmentvirtualids = "";// 虚拟部门id;
			boolean falg = false;

			//验证用户名
			String loginid = (String) getValue("loginid");
			baseData.remove("loginid");
			if ("".equals(loginid) && accounttype == 1) {
				rs.executeSql("select loginid from HrmResource where id =" + belongto);
				if (rs.next()) {
					loginid = rs.getString("loginid");
				}
				if (!loginid.equals("")) {
					String maxidsql = "select max(id) as id from HrmResource where loginid like '" + loginid + "%'";
					rs.executeSql(maxidsql);
					if (rs.next()) {
						loginid = loginid + (rs.getInt("id") + 1);
						falg = true;
					}
				}
			}

			String seclevel = (String) getValue("seclevel");
			if(Math.abs(Util.getIntValue(seclevel)) > 999){
				seclevel = "0";
				//this.reqManager.setMessage(SystemEnv.getHtmlLabelNames("518227", user.getLanguage()));
				setReqMessage(SystemEnv.getHtmlLabelName(518227, Util.getIntValue(user.getLanguage())));
				throw new Exception("安全级别区间错误！");
			}
			String managerstr = "";

			int managerid = Util.getIntValue(Util.null2String(getValue("managerid")), 0);
			String sql = "select managerstr, seclevel from HrmResource where id = " + managerid;
			rs.executeSql(sql);
			while (rs.next()) {
				String tmp_managerstr = rs.getString("managerstr");
				/*********处理managerstr 不以逗号开始或者结束的情况 形如 managerstr：8 begin***********/
				if (!tmp_managerstr.startsWith(",")) tmp_managerstr = "," + tmp_managerstr;
				if (!tmp_managerstr.endsWith(",")) tmp_managerstr = tmp_managerstr + ",";
				/*********处理managerstr 不以逗号开始或者结束的情况 形如 managerstr：8 end ***********/
				managerstr += tmp_managerstr;
				managerstr = "," + managerid + managerstr;
				managerstr = managerstr.endsWith(",") ? managerstr : (managerstr + ",");
				//seclevel = "".equals(seclevel) ? rs.getString("seclevel") : seclevel;
			}

			String departmentid = (String) getValue("departmentid");
			String subcmpanyid1 = new DepartmentComInfo().getSubcompanyid1(departmentid);

			if (!"".equals(loginid)) {
				rs.execute("select id from HrmResource where loginid='" + loginid + "'");
				if (rs.next()) {
					loginid = "";
					//this.reqManager.setMessage(SystemEnv.getHtmlLabelNames("412,18082", user.getLanguage()));
					setReqMessage(SystemEnv.getHtmlLabelName(10000165, Util.getIntValue(user.getLanguage())));
					throw new Exception("人员登录名重复！");
				} else if(accounttype != 1){
					if (ln.CkHrmnum() >= 0){
						setReqMessage(SystemEnv.getHtmlLabelName(16078, Util.getIntValue(user.getLanguage())));
						throw new Exception("有效用户数已到license上限！");
					}
					int limitUsers = 0;
					rs.execute("select limitUsers from HrmSubCompany where id=" + subcmpanyid1);
					if (rs.next()) {
						limitUsers = StringUtil.parseToInt(rs.getString("limitUsers"), 0);
					}
					if (limitUsers > 0) {
						int subIdInt = Util.getIntValue(subcmpanyid1, 0);
						int cnt = new HrmResourceManager().count(0, subIdInt);
						if (cnt >= limitUsers) {
							loginid = "";
							setReqMessage(SystemEnv.getHtmlLabelName(10000166, Util.getIntValue(user.getLanguage())));

							throw new Exception("人员超过分部设置的人员限制数！");
						}
					}
				}
			}

			//处理系统字段数据
			RecordSetTrans rst = new RecordSetTrans();
			rst.setAutoCommit(false);
			String para = "";

			String email = (String) getValue("email");
			if (!"".equals(email) && !EmailValidator.getInstance().isValid(email)) {
				//this.reqManager.setMessage(SystemEnv.getHtmlLabelNames("20869,24944", user.getLanguage()));
				setReqMessage(SystemEnv.getHtmlLabelName(10000167, Util.getIntValue(user.getLanguage())));
				throw new Exception("邮箱格式错误！");
			}


			String certificatenum = Util.null2String(baseData.get("certificatenum")).trim();
			if(!"".equals(certificatenum)){
				rs.executeQuery("select id from HrmResource where certificatenum = ? "+(new ServiceUtil().needCheckDismiss("certificatenum")?"":" and status in(0,1,2,3) "), certificatenum);
				if (rs.next()) {
					certificatenum = "";
					setReqMessage(SystemEnv.getHtmlLabelName(83521,weaver.general.Util.getIntValue(user.getLanguage())));
					throw new Exception("身份证重复！");
				}
			}

			//验证编号
			String workcode = (String) getValue("workcode");
			int jobtitle = Util.getIntValue(Util.null2String(getValue("jobtitle")), 0);
			workcode = CodeRuleManager.getCodeRuleManager().generateRuleCode(RuleCodeType.USER, subcmpanyid1, departmentid, jobtitle + "", workcode);
			if (!workcode.equals("")) {
				rs.execute("select workcode from HrmResource where workcode ='" + workcode + "'");
				if (rs.next()) {
					workcode = "";
					//this.reqManager.setMessage(SystemEnv.getHtmlLabelNames("714,18082", user.getLanguage()));
					setReqMessage(SystemEnv.getHtmlLabelName(503017, Util.getIntValue(user.getLanguage())));
					throw new Exception("编号重复！");
				}
			}

			rs.executeProc("HrmResourceMaxId_Get", "");
			rs.next();
			String id = String.valueOf(rs.getInt(1));
			String dsporder = String.valueOf(id);

			try {
				String sex = (String) getValue("sex");
				int joblevel = Util.getIntValue(Util.null2String(getValue("joblevel")), 0);
				int locationid = Util.getIntValue(Util.null2String(getValue("locationid")), 0);
				int status = Util.getIntValue(Util.null2String(getValue("status")), 0);
				int jobcall = Util.getIntValue(Util.null2String(getValue("jobcall")), 0);
				int assistantid = Util.getIntValue(Util.null2String(getValue("assistantid")), 0);
				String jobactivitydesc = (String) getValue("jobactivitydesc");
				String workroom = (String) getValue("workroom");
				String mobilecall = (String) getValue("mobilecall");
				String fax = (String) getValue("fax");
				int systemlanguage = Util.getIntValue(Util.null2String(getValue("systemlanguage")), 7);

				int mobileshowtype = 0;
				mobileshowtype = StringUtil.parseToInt(settings.getMobileShowTypeDefault(), 0);
				String costcenterid = "";
				para = "" + id + separator + workcode + separator + lastname + separator + sex + separator + resourceimageid
						+ separator + departmentid + separator + costcenterid + separator + jobtitle + separator
						+ joblevel + separator + jobactivitydesc + separator + managerid + separator + assistantid + separator
						+ status + separator + locationid + separator + workroom + separator + telephone + separator
						+ mobile + separator + mobilecall + separator + fax + separator + jobcall + separator
						+ subcmpanyid1 + separator + managerstr + separator + accounttype + separator + belongto + separator
						+ systemlanguage + separator + email + separator + dsporder + separator + mobileshowtype;
				writeLog("in HrmResourceEntrantAction HrmResourceBasicInfo_Insert ： " + para);
				rst.executeProc("HrmResourceBasicInfo_Insert", para);
				if (locationid > 0)
					rst.executeSql("update hrmresource set countryid=(select countryid from HrmLocations where id=" + locationid + ") where id=" + id);
				if (!"".equals(loginid)) {
					if ("".equals(password)) {
						password = "1";
					}
					String[] pwds = PasswordUtil.encrypt(password);

					String logidsql = "update HrmResource set loginid = ?,password =?,salt=? where id =?";
					rst.executeUpdate(logidsql, loginid, pwds[0], pwds[1], id);
				}
				if (seclevel == null || "".equals(seclevel)) {
					seclevel = "0";
				}
				rst.executeUpdate("update hrmresource set seclevel = ?, "+ DbFunctionUtil.getInsertUpdateSetSql(rs.getDBType(),creater) +" where id=? ", seclevel, id);
				//String p_para = id + separator + departmentid + separator + subcmpanyid1 + separator + managerid + separator + seclevel + separator + managerstr + separator + "0" + separator + "0" + separator + "0" + separator + "0" + separator + "0" + separator + "0";

				//System.out.println(p_para);
				//rst.executeProc("HrmResourceShare", p_para);
				rst.commit();

				String messagerurl = (String) getValue("messagerurl");
				messagerurl = getImageIdByDocid(messagerurl);
				//考虑到流程文档权限的问题，把附件复制一份
				messagerurl = Util.null2String(ImageFileManager.copyImageFile(Util.getIntValue(messagerurl)));
				if (!"".equals(messagerurl) && !"0".equals(messagerurl)) {
					messagerurl = copyFile(messagerurl);
					rs.executeUpdate("update hrmresource set messagerurl = ? where id = ?", messagerurl, id);
				}

				try {
					HrmFaceCheckManager.setUserPassowrd(id+"",password);
					new ResourceComInfo().removeResourceCache();
				} catch (Exception e) {
					writeLog(e);
				}

				HrmFaceCheckManager.sync(id, HrmFaceCheckManager.getOptInsert(), this.getClass().getName(), HrmFaceCheckManager.getOaResource());

			} catch (Exception e) {
				rst.rollback();
				writeLog(e);
				throw e;
			}

			para = "" + id;

			int userid = creater;
			Calendar todaycal = Calendar.getInstance();
			String today = Util.add0(todaycal.get(Calendar.YEAR), 4) + "-" + Util.add0(todaycal.get(Calendar.MONTH) + 1, 2) + "-" + Util.add0(todaycal.get(Calendar.DAY_OF_MONTH), 2);
			String userpara = "" + userid + separator + today;

			for (int i = 0; i < 5; i++) para += separator + "" + separator + "" + separator + "" + separator + "";
			rs.executeProc("HrmResourceDefine_Update", para);

			rs.executeProc("HrmResource_CreateInfo", "" + id + separator + userpara + separator + userpara);

			// 改为只进行该人缓存信息的添加
			new ResourceComInfo().addResourceInfoCache(id);

			new SalaryManager().initResourceSalary(id);

			para = "" + id + separator + managerid + separator + departmentid + separator + subcmpanyid1 + separator + "0" + separator + managerstr;
			rs.executeProc("HrmResource_Trigger_Insert", para);

			String sql_1 = ("insert into HrmInfoStatus (itemid,hrmid) values(1," + id + ")");
			rs.executeSql(sql_1);
			String sql_2 = ("insert into HrmInfoStatus (itemid,hrmid) values(2," + id + ")");
			rs.executeSql(sql_2);
			String sql_3 = ("insert into HrmInfoStatus (itemid,hrmid) values(3," + id + ")");
			rs.executeSql(sql_3);
			String sql_10 = ("insert into HrmInfoStatus (itemid,hrmid) values(10," + id + ")");
			rs.executeSql(sql_10);

			String name = lastname;

			String SWFAccepter = "";
			String SWFTitle = "";
			String SWFRemark = "";
			String Subject = "";
			Subject = SystemEnv.getHtmlLabelName(15670, user.getLanguage());
			Subject += ":" + name;

			if ("1".equals(settings.getEntervalid())) {//入职提醒
/*		    String thesql="select distinct hrmid from HrmInfoMaintenance where id<4 or id = 10";
	rs.executeSql(thesql);
	String members="";
	while(rs.next()){
		int hrmid_tmp = Util.getIntValue(rs.getString("hrmid"));//TD9392
		if(hrmid_tmp > 0 && creater != hrmid_tmp){
			members += ","+rs.getString("hrmid");
		}
	}
if(!members.equals("")){
		members = members.substring(1);
		SWFAccepter=members;*/


				String thesql = "select  hrmids from HrmInfoMaintenance where id<4 or id = 10";
				rs.executeSql(thesql);
				String members = "";
		/*while(rs.next()){
			int hrmid_tmp = Util.getIntValue(rs.getString("hrmids"));//TD9392
			if(hrmid_tmp > 0 && user.getUID() != hrmid_tmp){
				members += ","+rs.getString("hrmids");
			}
		}*/
				while (rs.next()) {
					String hrmid_tmp = Util.null2String(rs.getString("hrmids"));//TD9392
					if (hrmid_tmp.length() != 0) {
						members += "," + rs.getString("hrmids");
					}
				}
				if (!members.equals("")) {
					members = members.substring(1);
					members = duplicateRemoval(members, user.getUID() + "");
					SWFAccepter = members;
					SWFTitle = SystemEnv.getHtmlLabelName(15670, user.getLanguage());
					SWFTitle += ":" + name;
					SWFTitle += "-" + createrName;
					SWFTitle += "-" + today;
//		        SWFRemark="<a href=/hrm/employee/EmployeeManage.jsp?hrmid="+id+">"+Util.fromScreen2(Subject,user.getLanguage())+"</a>";
					SWFRemark = "<a class='wea-hrm-new-employee-set'  onClick=\"openHrmNewEmployeeSetDialog(" + id + ")\"  style=\"cursor:pointer;\"  id = '" + id + "'>" + Util.fromScreen2(Subject, user.getLanguage()) + "</a>";
                RemindWorkflowThreadUtil.remindWorkflowPool.submit(new RemindWorkflowThread(SWFTitle, 0, creater, SWFAccepter, SWFRemark));
				}
			}
			//更新虚拟组织部门id
			if (departmentvirtualids.length() > 0) {
				//保存前先删除需要删除的数据，因为有managerid 所以不能全部删除再保存
				sql = "delete from hrmresourcevirtual where resourceid=" + id + " and departmentid not in (" + departmentvirtualids + ")";
				rs.executeSql(sql);

				String[] departmentvirtualid = departmentvirtualids.split(",");
				for (int i = 0; departmentvirtualid != null && i < departmentvirtualid.length; i++) {
					rs.executeSql(" select count(*) from HrmResourceVirtual where departmentid ='" + departmentvirtualid[i] + "' and resourceid = " + id);
					if (rs.next()) {
						//如果已存在 无需处理
						if (rs.getInt(1) > 0) continue;
					}

					//写入
					int tmpid = 0;
					rs.executeSql("select max(id) from HrmResourceVirtual ");
					if (rs.next()) {
						tmpid = rs.getInt(1) + 1;
					}
					String subcompanyid = new DepartmentVirtualComInfo().getSubcompanyid1(departmentvirtualid[i]);
					sql = " insert into HrmResourceVirtual (id,resourceid,subcompanyid,departmentid ) " + " values (" + tmpid + "," + id + "," + subcompanyid + "," + departmentvirtualid[i] + ")";
					rs.executeSql(sql);
				}
			}

			if (!"".equals(lastname)) {
				String quickSearchStr = new HrmCommonServiceImpl().generateQuickSearchStr(lastname);
				String sqlStr = "update HrmResource set pinyinlastname = ?, ecology_pinyin_search = ? where id = ?";
				rs.executeUpdate(sqlStr, quickSearchStr, quickSearchStr, id);
			}

			SysMaintenanceLog log = new SysMaintenanceLog();
			log.resetParameter();
			log.setRelatedId(Util.getIntValue(id));
			log.setRelatedName(lastname);
			log.setOperateItem("29");
			log.setOperateUserid(creater);
			log.setClientAddress(clientIp);
			log.setOperateType("1");
			log.setOperateDesc("HrmResourceBasicInfo_Insert");
			log.setSysLogInfo();


			//个人信息
			String tempcertificatenum = certificatenum;
			if (!certificatenum.equals("")) {
				rs.executeSql("select id from HrmResource where id<>" + id + " and certificatenum='" + certificatenum + "'");
				if (rs.next()) {
					rs.executeSql("select certificatenum from HrmResource where id=" + id);
					if (rs.next()) {
						tempcertificatenum = Util.null2String(rs.getString("certificatenum"));
					}
				}
			}

			String birthday = (String) getValue("birthday");
			String folk = (String) getValue("folk");
			String nativeplace = (String) getValue("nativeplace");
			String regresidentplace = (String) getValue("regresidentplace");
			String maritalstatus = (String) getValue("maritalstatus");
			String policy = (String) getValue("policy");
			String bememberdate = (String) getValue("bememberdate");
			String bepartydate = (String) getValue("bepartydate");
			String islabouunion = (String) getValue("islabouunion");
			String educationlevel = (String) getValue("educationlevel");
			String degree = (String) getValue("degree");
			String healthinfo = (String) getValue("healthinfo");
			String height = (String) getValue("height");
			String weight = (String) getValue("weight");
			String residentplace = (String) getValue("residentplace");
			String homeaddress = (String) getValue("homeaddress");
			String tempresidentnumber = (String) getValue("tempresidentnumber");
			para = "" + id + separator + birthday + separator + folk + separator + nativeplace + separator
					+ regresidentplace + separator + maritalstatus + separator + policy + separator + bememberdate + separator
					+ bepartydate + separator + islabouunion + separator + educationlevel + separator + degree + separator
					+ healthinfo + separator + height + separator + weight + separator + residentplace + separator
					+ homeaddress + separator + tempresidentnumber + separator + tempcertificatenum;

			rs.executeProc("HrmResourcePersonalInfo_Insert", para);
			rs.executeProc("HrmResource_ModInfo", "" + id + separator + userpara);

			log.resetParameter();
			log.setRelatedId(Util.getIntValue(id));
			log.setRelatedName(new ResourceComInfo().getResourcename(id));
			log.setOperateItem("29");
			log.setOperateUserid(creater);
			log.setClientAddress(clientIp);
			log.setOperateType("1");
			log.setOperateDesc("HrmResourcePersonalInfo_Insert");
			log.setSysLogInfo();

			//工作信息
			String usekind = (String) getValue("usekind");
			String startdate = (String) getValue("startdate");
			String probationenddate = (String) getValue("probationenddate");
			String enddate = (String) getValue("enddate");
			para = "" + id + separator + usekind + separator + startdate + separator + probationenddate + separator + enddate;
			rs.executeProc("HrmResourceWorkInfo_Insert", para);
			rs.executeProc("HrmResource_ModInfo", "" + id + separator + userpara);

			log.resetParameter();
			log.setRelatedId(Util.getIntValue(id));
			log.setRelatedName(new ResourceComInfo().getResourcename(id));
			log.setOperateItem("29");
			log.setOperateUserid(creater);
			log.setClientAddress(clientIp);
			log.setOperateType("1");
			log.setOperateDesc("HrmResourceWorkInfo_Insert");
			log.setSysLogInfo();

			baseData.remove("jobactivity");
			baseData.remove("jobGroupId");
			if (baseData.keySet().size() > 0) {
				sql = "update hrmresource set ";
				String sqlWhere = " where id = " + id;
				String setSql = "";
				for (String key : baseData.keySet()) {
					setSql += ", " + key + " = ";
					Map<String, Object> fieldConfig = fieldMap.get(key);
					String fieldDbType = (String) fieldConfig.get("FieldDbType");
					String type = Util.null2String(fieldConfig.get("Type"));
					String htmltype = Util.null2String(fieldConfig.get("HtmlType"));
					String value = Util.null2String(baseData.get(key));
					if (htmltype.equals("6")) {
						String fileid = value;
						List<String> transId = new ArrayList<>();
						List<String> ids = Util.splitString2List(fileid, ",");
						for (String fid : ids) {
							String copyId = getImageIdByDocid(fid);
							//考虑到流程文档权限的问题，把附件复制一份
							copyId = Util.null2String(ImageFileManager.copyImageFile(Util.getIntValue(copyId)));
							if(Util.getIntValue(copyId, 0)>0)
							transId.add(copyId);
						}
						setSql += (transId.size() == 0 ? "null" : ("'" + String.join(",", transId)) + "'");
					} else if (fieldDbType.startsWith("text") || fieldDbType.startsWith("char") ||
							fieldDbType.startsWith("varchar") || type.equals("161") || type.equals("162") || type.equals("257") || type.equals("256")) {
						setSql += !"".equals(value) ? "'" + value + "'" : "null";
					} else {
						if (value.equals("")) {
							setSql += "null";
						} else {
							setSql += value;
						}
					}
				}
				if (!"".equals(setSql)) {
					setSql = setSql.substring(1);
					sql += setSql + sqlWhere;
					rs.executeUpdate(sql);
				}
			}
			updateWorkInfo(id);

			rs.executeUpdate("update hrmresource set " + DbFunctionUtil.getInsertUpdateSetSql(rst.getDBType(), user.getUID()) + " where id=" + id);

			//保存自定义字段数据
			insertCustData(log, id);
			ResourceComInfo resourceComInfo = new ResourceComInfo();
			try{
				//登录信息签名
				PasswordUtil.saveSign(""+id);
			}catch (Exception e){
				writeLog(e);
			}

			try{

				String companystartdate = "";
				String formsql = "select companystartdate from HrmResource where id = " + id;
				System.out.println("requestId:"+requestId+":id:"+id+":formsql:"+formsql);
				rs.executeQuery(formsql);
				if(rs.next()){
					companystartdate = Util.null2s(rs.getString("companystartdate"),"");
				}
				System.out.println("requestId:"+requestId+":id:"+id+":companystartdate:"+companystartdate);
				if(companystartdate != null && companystartdate.length() > 0){
					String changefirstdate = DateUtil.getFirstDayOfMonth(companystartdate);
					String endlastdate = DateUtil.getCurrentDate();
					String formInsertSql = "insert into kq_date_change_log(resourceid,changefromdate,changetodate,status) values(?,?,?,0) ";
					boolean isok = rs.executeUpdate(formInsertSql, id,changefirstdate,endlastdate);
					System.out.println("requestId:"+requestId+":id:"+id+":changefirstdate:"+changefirstdate+":endlastdate:"+endlastdate+":isok:"+isok);
				}
			}catch (Exception e){
				writeLog(e);
			}

		}catch (Exception ex){
			writeLog(ex);
			throw ex;
		}
	}

	private Object transValue(String key){
		Object value = Util.null2String(rsdt.getString(key));
//		Map<Integer, List> optionsMap = (key.indexOf("custField_") > -1 ? cusSelectItem : hrmSelectItem);
//		try{
//			if(fieldMap.containsKey(key)) {
//				Map<String, Object> fieldConfig = fieldMap.get(key);
//				String htmlType = Util.null2String(fieldConfig.get("HtmlType"));
//				Integer fieldId = Util.getIntValue(Util.null2String(fieldConfig.get("id")));
//				if (!"".equals(value.toString()) && htmlType.equals("5") && hrmSelectItem.containsKey(fieldId)) {
//					List<Map> options = optionsMap.get(fieldId);
//					value = options.get(Integer.parseInt(value.toString())).get("selectValue");
//				}
//			}
//		}catch (Exception ex){
//			writeLog(key, ex);
//		}
		return value;
	}

	private Object getValue(String key) throws Exception{
		if(baseData.containsKey(key)){
			Object value = baseData.get(key);
			baseData.remove(key);

			return value;
		}else
			return "";
	}

	private Object getCustValue(String key) throws Exception{
		if(custData.containsKey(key)){
			Object value = custData.get(key);
			custData.remove(key);

			return value;
		}else
			return "";
	}

	private void updateWorkInfo(String id){
		SimpleDateFormat sdf = new SimpleDateFormat("YYYY-MM-DD");
		DecimalFormat df = new DecimalFormat("0.00");
		String companyworkyear = "";
		String workyear = "";

		try {
			String companystartdate = (String)getValue("companystartdate");
			String workstartdate = (String)getValue("workstartdate");

			Date today = Calendar.getInstance().getTime();
			Date companyStartDate = null;
			Date workStartDate = null;

			Long to = 0L;
			Long from = 0L;

			if (!"".equals(companystartdate)) {
				companyStartDate = sdf.parse(companystartdate);

				to = today.getTime();
				from = companyStartDate.getTime();

				if(to > from){
					companyworkyear = df.format((float)(to - from) / (24*3600*1000*365.0));
				}

			}
			if(!"".equals(workstartdate)){
				workStartDate = sdf.parse(workstartdate);
				to = today.getTime();
				from = workStartDate.getTime();

				if(to > from){
					workyear = df.format((float)(to - from) / (24*3600*1000*365.0));
				}
			}

			List<String> lsParams = new ArrayList<>();
			lsParams.add(companyworkyear.equals("") ? null : companyworkyear);
			lsParams.add(workyear.equals("") ? null : workyear);
			lsParams.add(id);
			String sql = " update hrmresource set companyworkyear=?,workyear=? where id=?";
			new RecordSet().executeUpdate(sql, lsParams);
			HrmDateCheck hrmDateCheck = new HrmDateCheck();
			hrmDateCheck.calWorkInfo(id);
		}catch (Exception ex){
			ex.printStackTrace();
		}
	}

	private void insertCustData(SysMaintenanceLog log, String id) throws Exception{
		editCustomDataE9Add(Util.getIntValue(id));

		log.resetParameter();
		log.setRelatedId(Util.getIntValue(id));
		log.setRelatedName(new ResourceComInfo().getResourcename(id));
		log.setOperateItem("29");
		log.setOperateUserid(creater);
		log.setClientAddress(clientIp);
		log.setOperateType("1");
		log.setOperateDesc("HrmResourceCustomerInfo_Insert");
		log.setSysLogInfo();
	}

	/**
	 * 编辑自定义数据
	 * @param dataId
	 */
	private void editCustomDataE9Add(int dataId) throws  Exception {
		String scope = "HrmCustomFieldByInfoType";
		List<Integer> scopeList = new ArrayList<Integer>(Arrays.asList(-1, 1, 3));
		/*RecordSet rs = new RecordSet();
		List<Integer> scopeIds = new ArrayList<>();
		for(Integer parentId : scopeList){
			rs.executeQuery("select id from CUS_TREEFORM where scope = 'HrmCustomFieldByInfoType' and PARENTID = ? or id = ?", parentId, parentId);
			while (rs.next()){
				scopeIds.add(rs.getInt("id"));
			}
		}*/

		for(Integer scopeId : scopeList){
			addCustomDataE9Add(scope, scopeId, dataId);
		}
	}

	/**
	 * 添加自定义数据
	 * @param dataId
	 */
	private void addCustomDataE9Add(String scope, int scopeId, int dataId) throws Exception {
		RecordSet rs = new RecordSet();
		CustomFieldManager cfm = new CustomFieldManager(scope, scopeId);
		cfm.getCustomFields();
		String sql = "insert into cus_fielddata";
		String nameStr = "";
		String valueStr = "";
		String prefix = "custField_" + (scopeId < 0 ? "_" : "") + Math.abs(scopeId) + "_";
		while (cfm.next()) {
			nameStr += ",field" + cfm.getId();
			String value = Util.null2String(getCustValue(prefix + cfm.getFieldDBName()));
			if(cfm.getHtmlType().equals("6")) {
				String fileid = Util.null2String(value);
				List<String> transId = new ArrayList<>();
				List<String> ids = Util.splitString2List(fileid, ",");
				for(String fid : ids){
					String copyId = getImageIdByDocid(fid);
					//考虑到流程文档权限的问题，把附件复制一份
					copyId = Util.null2String(ImageFileManager.copyImageFile(Util.getIntValue(copyId)));
					if(Util.getIntValue(copyId, 0)>0)
					transId.add(copyId);
				}
				valueStr += "," + (transId.size() == 0 ? "null" : "'" + String.join(",", transId) + "'");
			} else if (cfm.getFieldDbType().startsWith("text") || cfm.getFieldDbType().startsWith("char") ||
					cfm.getFieldDbType().startsWith("varchar")||cfm.getType()==161||cfm.getType()==162 || cfm.getType()==257 || cfm.getType()==256) {
				valueStr += "," + (!"".equals(value) ? "'" + value + "'" : "null");
			} else {
				if (value.equals("")) {
					valueStr += ",null";
				} else {
					valueStr += "," + value;
				}
			}
		}
		if (!nameStr.equals("")) {
			nameStr = nameStr.substring(1);
			valueStr = valueStr.substring(1);

			sql += "(scope,scopeid,id," + nameStr + ") values('" + scope + "'," + scopeId + "," + dataId + "," + valueStr + ")";

			RecordSet rsSel = new RecordSet();
			rsSel.executeQuery("select id from cus_fielddata where scope=? and scopeid=? and id=?", scope, scopeId, dataId);
//			rsSel.executeSql("select id from cus_fielddata where scope='" + scope + "' and scopeid=" + scopeId + " and id=" + dataId);
			if (rsSel.next()) {//查询是否已存在数据

			}else{
				rs.executeSql(sql);
			}
		}
	}

	private void findFields() throws Exception{
		baseData.clear();
		custData.clear();
		fieldMap.clear();

		baseData.put("loginid", "");
		baseData.put("password", "");
		baseData.put("seclevel", "");

		List<Integer> scopeList = new ArrayList<Integer>(Arrays.asList(-1, 1, 3));
		RecordSet rs = new RecordSet();
		for(Integer parentId : scopeList){
			rs.executeQuery("select id from CUS_TREEFORM where scope = 'HrmCustomFieldByInfoType' and PARENTID = ? or id = ?", parentId, parentId);
			while(rs.next()){
				int scopeId = rs.getInt("id");
				HrmFieldManager hfm = new HrmFieldManager("HrmCustomFieldByInfoType", scopeId);
				hfm.getCustomFields();
				while (hfm.next()) {
					try {
						String fieldName = hfm.getFieldname();
						if (!hfm.isUse() || donotExportFields.contains(fieldName))
							continue;
						if(hfm.isBaseField(fieldName)){
							baseData.put(fieldName, "");
						}else{
							fieldName = "custField_" + (scopeId < 0 ? "_" : "") + Math.abs(scopeId) + "_" + fieldName;
							custData.put(fieldName, "");
						}
						fieldMap.put(fieldName, new HashMap());
						fieldMap.get(fieldName).put("id", hfm.getFieldid());
						fieldMap.get(fieldName).put("FieldDbType", hfm.getFieldDbType());
						fieldMap.get(fieldName).put("Type", hfm.getType());
						fieldMap.get(fieldName).put("HtmlType", hfm.getHtmlType());
					} catch (Exception ex) {
						writeLog(ex);
						throw ex;
					}
				}
			}
		}
	}


	/**
	 *
	 * @param resourceimageid
	 * @return imageid
	 */
	private String getImageIdByDocid(String resourceimageid) {
		String imageid = "0";
		if(Util.getIntValue(Util.null2String(resourceimageid), 0)>0){
			RecordSet rs1 = new RecordSet();
			String imageSql = "select imagefileid from DocImageFile where docid=?";
			rs1.executeQuery(imageSql, resourceimageid);
			rs1.next();
			imageid = StringUtil.vString(rs1.getString("imagefileid"),"0");
		}
		return imageid;
	}


	@SuppressWarnings({ "unchecked", "rawtypes" })
	public String duplicateRemoval(String str,String userid) {
		String[]  ss = str.split(",");
		List<String> list =  new ArrayList<String>();
		for (String s : ss) {
			list.add(s.trim());
		}
		HashSet hs  =   new  HashSet(list);
		list.clear();
		list.addAll(hs);
		if(list.contains(userid))list.remove(userid);
		str=list.toString().replace("[", "").replace("]", "").replace(" ", "");
		return  str ;
	}

	private String copyFile(String messagerurl){
		try {
			writeLog("messagerurl:" + messagerurl);
			String filePath = "messager/usericon/" + UUID.randomUUID().toString().replace("-", "").toLowerCase() + ".jpg";
			ImageFileManager imageFileManager = new ImageFileManager();
			imageFileManager.getImageFileInfoById(Util.getIntValue(messagerurl));
			InputStream inputStream = imageFileManager.getInputStream();
			String filesize = imageFileManager.getFilesize();

			FileOutputStream out = new FileOutputStream(new File(GCONST.getRootPath() + filePath));

			byte[] buffer = new byte[Util.getIntValue(filesize)];
			int len;

			while ((len = inputStream.read(buffer)) > 0) {
				out.write(buffer, 0, len);
			}

			out.close();
			inputStream.close();

			return "/" + filePath;
		}catch (Exception ex){
			writeLog(ex);
			return "";
		}
	}

	private void loadHrmSelectItem(){
		try{
			hrmSelectItem.clear();
			RecordSet rs = new RecordSet();
			rs.executeQuery("select fieldid, selectvalue, listorder from hrm_selectitem");
			while(rs.next()){
				int fieldId = rs.getInt("fieldid");
				String selectValue = rs.getString("selectvalue");
				String fieldOrder = Util.null2String(rs.getString("listorder"));
				if("".equals(fieldOrder))
					fieldOrder = selectValue;
				Map<String, Object> data = new HashMap<>();
				data.put("fieldId", fieldId);
				data.put("selectValue", selectValue);
				data.put("fieldOrder", fieldOrder);
				if(!hrmSelectItem.containsKey(fieldId))
					hrmSelectItem.put(fieldId, new ArrayList());
				hrmSelectItem.get(fieldId).add(data);
			}
			for(Integer key : hrmSelectItem.keySet()){
				hrmSelectItem.put(key, sortData(hrmSelectItem.get(key)));
			}
		}catch (Exception ex){
			writeLog(ex);
		}
	}

	private void loadCusSelectItem(){
		try{
			cusSelectItem.clear();
			RecordSet rs = new RecordSet();
			rs.executeQuery("select fieldid, selectvalue, fieldorder from cus_selectitem");
			while(rs.next()){
				int fieldId = rs.getInt("fieldid");
				String selectValue = rs.getString("selectvalue");
				String fieldOrder = Util.null2String(rs.getString("fieldorder"));
				if("".equals(fieldOrder))
					fieldOrder = selectValue;
				Map<String, Object> data = new HashMap<>();
				data.put("fieldId", fieldId);
				data.put("selectValue", selectValue);
				data.put("fieldOrder", fieldOrder);
				if(!cusSelectItem.containsKey(fieldId))
					cusSelectItem.put(fieldId, new ArrayList());
				cusSelectItem.get(fieldId).add(data);
			}
			for(Integer key : cusSelectItem.keySet()){
				cusSelectItem.put(key, sortData(cusSelectItem.get(key)));
			}
		}catch (Exception ex){
			writeLog(ex);
		}
	}

	private List sortData(List list){
		Collections.sort(list, new Comparator<Map>() {
			@Override
			public int compare(Map o1, Map o2) {
				int a = Integer.parseInt(o1.get("fieldOrder").toString());
				int b = Integer.parseInt(o2.get("fieldOrder").toString());
				return a - b;
			}
		});
		return list;
	}

	private void setReqMessage(String context){
		this.reqManager.setMessagecontent(context);
	}
}
